/*
 * Copyright (c) 2007, Stefan Helbing.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: tftpd.c,v 1.0 2007/09/30 09:43:54 sth Exp $
 *
 */

#include "tftp.h"
#include <string.h>

uip_ipaddr_t tftp_server;

void tftp_init()
{
	struct uip_udp_conn *c = uip_udp_new(&tftp_server, HTONS(TFTP_UDP_PORT));
	if (c != NULL) {
		uip_udp_bind(c, HTONS(TFTP_UDP_PORT));
//		c->appstate.tftp.transmit_state = 0;
//		c->appstate.tftp.state = TFTP_STATE_WAIT;
//		c->appstate.tftp.timeout = 0;
#ifdef SYSLOG_TFTP
        syslog_message_P("getting file via tftp...");
#endif
	}
}


void tftp_recvfile(int fd, char *name, char *mode)
{
	struct tftphdr *tftp_header = (struct tftphdr *)uip_appdata;
	volatile int firsttrip = 1;
	volatile int size = 0;
	volatile u_int16_t block = 1;

	if (firsttrip) {
		size = tftp_makerequest(RRQ, name, tftp_header, mode);
		firsttrip = 0;
	} else {
		tftp_header->th_opcode = htons((u_short)ACK);
		tftp_header->th_block = htons((u_short)(block));
		size = 4;
		block++;
	}

	uip_udp_send(sizeof(struct tftphdr));

//	tftp_write_behind(file, convert);
}


int tftp_makerequest(int request, char *name, struct tftphdr *tp, char *mode)
{
	register char *cp;

	tp->th_opcode = htons((u_short)request);
	cp = tp->th_stuff;
	strcpy(cp, name);
	cp += strlen(name);
	*cp++ = '\0';
	strcpy(cp, mode);
	cp += strlen(mode);
	*cp++ = '\0';
	return (cp - (char *)tp);
}


//int tftp_write_behind(FILE *file, int convert)
//{
//	char *buf;
//	int count;
//	register int ct;
//	register char *p;
//	register int c;                 /* current character */
//	struct bf *b;
//	struct tftphdr *dp;
//
//	b = &bfs[nextone];
//	if (b->counter < -1)            /* anything to flush? */
//		return 0;               /* just nop if nothing to do */
//
//	count = b->counter;             /* remember byte count */
//	b->counter = BF_FREE;           /* reset flag */
//	dp = (struct tftphdr *)b->buf;
//	nextone = !nextone;             /* incr for next time */
//	buf = dp->th_data;
//
//	if (count <= 0) return -1;      /* nak logic? */
//
//	if (convert == 0)
//		return write(fileno(file), buf, count);
//
//	p = buf;
//	ct = count;
//	while (ct--) {                  /* loop over the buffer */
//	    c = *p++;                   /* pick up a character */
//	    if (prevchar == '\r') {     /* if prev char was cr */
//		if (c == '\n')          /* if have cr,lf then just */
//		   fseek(file, -1, 1);  /* smash lf on top of the cr */
//		else
//		   if (c == '\0')       /* if have cr,nul then */
//			goto skipit;    /* just skip over the putc */
//		/* else just fall through and allow it */
//	    }
//	    putc(c, file);
//skipit:
//	    prevchar = c;
//	}
//	return count;
//}
//
/*---------------------------------------------------------------------------*/
